/*
* $Id$
*
*      $Log$
*      Revision 1.15  1998/04/16 14:24:46  moritz
*      - Offizielle Version 0.76b1 hinzugefgt
*      - Erzeugt Message-ID, wenn keine da war bei Mail
*      - qp-De- und Enkodierung auch von X-Z- und X-ZC- Headerzeilen
*      - NOKOP wird korrekt behandelt: Keine Erzeugung von
*        verrterischen To: oder Cc: Headerzeilen, wenn es
*        vorhanden ist
*
*      Revision 1.14  1998/02/13 13:23:32  moritz
*      - split make targets 'mail' and 'news'
*      - added rcs Ids into main programs
*
*      Revision 1.13  1998/02/03 22:00:15  moritz
*      - compilierende, ungetestete Veriosn von 0.76MB01
*
*      Revision 1.12  1998/02/03 16:18:41  moritz
*      - merged diffs from 0.75b2 to 0.76 into MB Version
*
*      Revision 1.11  1998/02/02 16:29:28  moritz
*      - Aenderungen der offiziellen Version bis 0.75b2 eingearbeitet
*
*      Revision 1.10  1996/09/19 15:35:51  moritz
*      Aenderungen von Matthais Watermann eingebaut:
*      - Pfade in Makefiles
*      - Logfiles mit Byte-Angaben
*      - ZNETZ-TEXT loeschen
*
*      Revision 1.10  1996/09/06 12:51:37  matthias
*      'trap' fuer Linux ausgeschaltet;
*
*      #ifndef ISLINUXMW
*      init_trap(argv[0]);
*      #endif
*/
/*
 *  UNIX-Connect, a ZCONNECT(r) Transport and Gateway/Relay.
 *  Copyright (C) 1993-94  Martin Husemann
 *  Copyright (C) 1995     Christopher Creutzig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * ------------------------------------------------------------------------
 * Eine deutsche Zusammenfassung zum Copyright dieser Programme sowie der
 * GNU General Public License finden Sie in der Datei README.1st.
 * ------------------------------------------------------------------------
 *
 *  Bugreports, suggestions for improvement, patches, ports to other systems
 *  etc. are welcome. Contact the maintainer by e-mail:
 *  christopher@nescio.foebud.org or snail-mail:
 *  Christopher Creutzig, Im Samtfelde 19, 33098 Paderborn
 *
 *  There is a mailing-list for user-support:
 *   unix-connect@mailinglisten.im-netz.de,
 *  write a mail with subject "Help" to
 *   unix-connect-request@mailinglisten.im-netz.de
 *  for instructions on how to join this list.
 */


/*
 * uurnews - liest einen NEWS-Batch und erzeugt eine ZCONNECT Datei
 *
 * (C) Copyright 1992 M. Husemann
 * (C) Copyright 1995 C. Creutzig
 */

#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#ifdef HAS_STRING_H
# include <string.h>
#endif
#ifdef HAS_STRINGS_H
# include <strings.h>
#endif
#include "lib.h"
#ifdef NEED_VALUES_H
#include <values.h>
#endif
#include <errno.h>
#ifndef NO_UNISTD_H
#include <unistd.h>
#endif
#include <ctype.h>
#ifdef HAS_BSD_DIRECT
#include <sys/types.h>
#include <sys/dirent.h>
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/stat.h>

#include "utility.h"
#include "header.h"
#include "uulog.h"
#include "hd_nam.h"
#include "hd_def.h"
#include "version.h"
#include "ministat.h"
#include "mime.h"
#include "trap.h"
#include "uuconv.h"

#ifdef APC_A2B
#include "apc_a2b.h"
#endif

const char rcs_id[] = "$Id$";

header_p uu_rd_para(FILE *);

extern int dont_gate;

/*
 *  Globale Variable fuer die aktuelle Batch-Laenge
 *  (wird auf 0 heruntergezaehlt)
 */
long readlength = 0;

void usage(void);
void convert(FILE *, FILE *);
void convdata(FILE *news, FILE *zconnect);

	
static char *bigbuffer	= NULL;
static char *readbuffer	= NULL;
long buffsize = 0;	/* gegenwaertig allokierte Buffer-Groesse */

extern char *netcalldir;

char *fqdn = NULL;
int main_is_mail = 0;	/* Nein, wir sind nicht fuer PM EMP:s zustaendig */

int main(int argc, char **argv)
{
	FILE *fin, *fout;
	int filter;
	char *datei;

#ifdef TRAPSIGNAL
       init_trap(argv[0]);
#endif
	filter = 0;
	ulibinit();	
	minireadstat();
	if (argc != 3) usage();
	if (strcmp(argv[1], "-f") == 0) {
		filter = 1;
		fin = stdin;
		fqdn = argv[2];
		
		datei = open_unique_spoolfile(&fout);
		if (!datei) {
			perror("open spool file");
			exit(1);
		}
	} else {
		fin = fopen(argv[1], "rb");
		fout = fopen(argv[2], "ab");
		datei=argv[2];
	}

	if (!fin) {
		perror(argv[1]);
		exit(1);
	}
	if (!fout) {
		perror(datei);
		exit(1);
	}
	convert(fin, fout);
	fclose(fout);

	if (filter) {
		if (!rename_spoolfile(datei, "brt")) {
			perror("rename spool file");
			exit (1);
		}
	}
	else {
		fclose(fin);
		remove(argv[1]);
	}

	return 0;
}

void usage(void)
{
	fputs(	"UUrnews  -  RFC1036 Batch nach ZCONNECT konvertieren\n"
		"Aufruf:\n"
		"        uurnews (NEWS-Datei) (ZCONNECT-Datei)\n"
		"oder    uurnews -f (FQDN-ZCONNECT-Host)\n"
		"           (News Batch von stdin)\n"
		"           (ZCONNECT in das entsprechende Netcall-Dir)\n"
		, stderr);
	exit(1);
}

void convert(FILE *news, FILE *zconnect)
{
	static char line[MAXLINE];

	while (!feof(news)) {
		 if (fgets(line, MAXLINE, news) == NULL) break;
		 readlength = -1;
		 sscanf(line, "#! rnews %ld", &readlength);
		 if (readlength < 1) {
		 	fprintf(stderr, "\n\nABBRUCH: Batch-Laenge \"%s\"\n", line);
			exit(1);
		 }
		 convdata(news, zconnect);
	}
}

void convdata(FILE *news, FILE *zconnect)
{
	char *n, *s;
	long msglen, wrlen;
	header_p hd, p;
	int binaer, eightbit;
	mime_header_info_struct mime_info;
	int ismime, decoded;
 
	msglen = 0;
	
	hd = uu_rd_para(news);

	/* MIME-Headerzeilen lesen */
	ismime = parse_mime_header(0, hd, &mime_info);

	if (readlength >= buffsize) {
		dfree(bigbuffer);
		dfree(readbuffer);
		buffsize = readlength+1000;
		bigbuffer = dalloc(buffsize*
#ifdef APC_A2B
					     4
#else
					     2
#endif
						);
 
		readbuffer = dalloc(buffsize);
	}

	/* Text lesen, LF nach CR/LF wandeln... */
	if (readlength > 0) {
		msglen = fread(readbuffer, 1, readlength, news);
		for (wrlen=0, s=readbuffer, n=bigbuffer; msglen; msglen--) {
			if (*s == '\n') {
				wrlen++;
				*n++ = '\r';
			}
			*n++ = *s++;
			wrlen++;
		}
	} else
		wrlen = 0;
	msglen=wrlen;
		
	/* Header konvertieren und ausgeben */
	hd = convheader(hd, zconnect, NULL, NULL);

	/* MIME Content-Transfer-Encoding decodieren */
	decoded = decode_cte(bigbuffer, &msglen, &eightbit, &mime_info);

	/* Wenn decodiert wurde, diese Headerzeile loeschen */
	if (decoded)
		hd = del_header(HD_UU_CONTENT_TRANSFER_ENCODING, hd);
	
	/* eightbit != 0 heisst, wir haben auch tatsaechlich-Daten.
	 * Eine Nachricht behandeln wir dann als binaer, wenn sie
	 * a) eine Mime-Nachricht ist, sie b) gerade eben tatsaechlich deco-
	 * diert wurde und sie c) keinen Text enthaelt.
	 */

	if (ismime && decoded && mime_info.type != cty_text && mime_info.type != cty_none)
		binaer=1;
	else
		binaer=0;

        /* MIME-Headerzeilen durchreichen? Eine schwere Frage.
         * Wir entscheiden uns so:
         */
	 
        if (ismime && (                         /* MIME-Nachricht */
             mime_info.encoding == cte_none ||  /* deren kodierung */
             decoded                            /* aufgeloest wurde */
           )
           && (
             mime_info.unixconnect_multipart || /* und die text/plain */
             mime_info.text_plain               /* oder spezial-Multi- */
           )                                    /* part ist */
         ) {
                hd = del_header(HD_UU_MIME_VERSION, hd);
                hd = del_header(HD_UU_CONTENT_TYPE, hd);
                hd = del_header(HD_UU_CONTENT_TRANSFER_ENCODING, hd);
        }
 
#ifdef UUCP_SERVER
	hd = del_header(HD_UU_XREF, hd);
	if (!dont_gate)
	   for (p=hd; p; p=p->next) {
		fprintf (zconnect, "U-%s: %s\r\n", p->header, p->text);
		while (p->other) {
			p = p->other;
			fprintf (zconnect, "U-%s: %s\r\n", p->header, p->text);
		}
	   }
#endif
#ifdef REAL_GATE
/* Artikel, die von Usenet-Seite aus hereinkommen, drfen nicht wieder
   herausgeschickt werden. */
	fprintf(zconnect, HN_X_DONT_GATE_IT": \r\n");
#endif
	free_para(hd);

        /* Jetzt sind alle Header-Angelegenheiten erledigt bis
         * auf FILE:, LEN:, TYP:, CHARSET: u. dgl.
         * Das ist fuer Mail und News ziemlich gleich,
         * es steht daher in uuconv.c
         */

        make_body(bigbuffer, msglen, &mime_info, binaer,
                readbuffer, zconnect);
 
        if (mime_info.filename)
                free(mime_info.filename);
}
